@@ -1,13 +1,12 @@
 import locale
-import traceback
 
 import numpy
 import numpy as np
 from PyQt5.QtCore import Qt, pyqtSlot
 from PyQt5.QtGui import QFontMetrics
-from PyQt5.QtWidgets import QInputDialog, QWidget, QUndoStack, QApplication
+from PyQt5.QtWidgets import QInputDialog, QWidget, QUndoStack, QApplication, QFileDialog
 
-from urh import constants
+from urh import settings
 from urh.controller.CompareFrameController import CompareFrameController
 from urh.controller.dialogs.ContinuousSendDialog import ContinuousSendDialog
 from urh.controller.dialogs.FuzzingDialog import FuzzingDialog
@@ -19,6 +18,8 @@ from urh.models.GeneratorTreeModel import GeneratorTreeModel
 from urh.plugins.NetworkSDRInterface.NetworkSDRInterfacePlugin import NetworkSDRInterfacePlugin
 from urh.plugins.PluginManager import PluginManager
 from urh.plugins.RfCat.RfCatPlugin import RfCatPlugin
+from urh.plugins.FlipperZeroSub.FlipperZeroSubPlugin import FlipperZeroSubPlugin
+from urh.signalprocessing.IQArray import IQArray
 from urh.signalprocessing.Message import Message
 from urh.signalprocessing.MessageType import MessageType
 from urh.signalprocessing.Modulator import Modulator
@@ -63,6 +64,10 @@ class GeneratorTabController(QWidget):
         self.rfcat_plugin = RfCatPlugin()
         self.init_rfcat_plugin()
 
+        # Flipper Zero Sub Plugin
+        self.set_FZSave_button_visibility()
+        self.flipperzerosub_plugin = FlipperZeroSubPlugin()
+
         self.modulation_msg_indices = []
 
         self.refresh_modulators()
@@ -101,7 +106,7 @@ class GeneratorTabController(QWidget):
 
     @property
     def total_modulated_samples(self) -> int:
-        return sum(int(len(msg.encoded_bits) * self.__get_modulator_of_message(msg).samples_per_bit + msg.pause)
+        return sum(int(len(msg.encoded_bits) * self.__get_modulator_of_message(msg).samples_per_symbol + msg.pause)
                    for msg in self.table_model.protocol.messages)
 
     @modulators.setter
@@ -156,6 +161,9 @@ class GeneratorTabController(QWidget):
         self.network_sdr_plugin.sending_stop_requested.connect(self.on_network_sdr_sending_stop_requested)
         self.network_sdr_plugin.current_send_message_changed.connect(self.on_send_message_changed)
 
+        # Flipper Zero
+        self.ui.btnFZSave.clicked.connect(self.on_btn_FZSave_clicked)
+
     @pyqtSlot()
     def refresh_tree(self):
         self.tree_model.beginResetModel()
@@ -212,7 +220,8 @@ class GeneratorTabController(QWidget):
             return
 
         modulator = self.modulators[0]
-        modulator.samples_per_bit = protocol.messages[0].bit_len
+        modulator.samples_per_symbol = protocol.messages[0].samples_per_symbol
+        modulator.bits_per_symbol = protocol.messages[0].bits_per_symbol
 
         if protocol.signal:
             modulator.sample_rate = protocol.signal.sample_rate
@@ -221,30 +230,22 @@ class GeneratorTabController(QWidget):
             if auto_freq is not None and auto_freq != 0:
                 modulator.carrier_freq_hz = auto_freq
 
+        modulator.parameters = modulator.get_default_parameters()
         self.show_modulation_info()
 
     def show_modulation_info(self):
         cur_ind = self.ui.cBoxModulations.currentIndex()
-        cur_mod = self.modulators[cur_ind]
-        self.ui.lCarrierFreqValue.setText(cur_mod.carrier_frequency_str)
-        self.ui.lCarrierPhaseValue.setText(cur_mod.carrier_phase_str)
-        self.ui.lBitLenValue.setText(cur_mod.bit_len_str)
-        self.ui.lSampleRateValue.setText(cur_mod.sample_rate_str)
-        mod_type = cur_mod.modulation_type_str
+        mod = self.modulators[cur_ind]
+        self.ui.lCarrierFreqValue.setText(mod.carrier_frequency_str)
+        self.ui.lCarrierPhaseValue.setText(mod.carrier_phase_str)
+        self.ui.lBitLenValue.setText(mod.samples_per_symbol_str)
+        self.ui.lSampleRateValue.setText(mod.sample_rate_str)
+        mod_type = mod.modulation_type
         self.ui.lModTypeValue.setText(mod_type)
-        if mod_type == "ASK":
-            prefix = "Amplitude"
-        elif mod_type == "PSK":
-            prefix = "Phase"
-        elif mod_type in ("FSK", "GFSK"):
-            prefix = "Frequency"
-        else:
-            prefix = "Unknown Modulation Type (This should not happen...)"
 
-        self.ui.lParamForZero.setText(prefix + " for 0:")
-        self.ui.lParamForZeroValue.setText(cur_mod.param_for_zero_str)
-        self.ui.lParamForOne.setText(prefix + " for 1:")
-        self.ui.lParamForOneValue.setText(cur_mod.param_for_one_str)
+        self.ui.lParamCaption.setText(mod.parameter_type_str)
+        self.ui.labelParameterValues.setText(mod.parameters_string)
+        self.ui.labelBitsPerSymbol.setText(str(mod.bits_per_symbol))
 
     def prepare_modulation_dialog(self) -> (ModulatorDialog, Message):
         preselected_index = self.ui.cBoxModulations.currentIndex()
@@ -255,11 +256,11 @@ class GeneratorTabController(QWidget):
                 selected_message = self.table_model.protocol.messages[min_row]
                 preselected_index = selected_message.modulator_index
             except IndexError:
-                selected_message = Message([1, 0, 1, 0, 1, 0, 1, 0], 0, [], MessageType("empty"))
+                selected_message = Message([1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0], 0, [], MessageType("empty"))
         else:
-            selected_message = Message([1, 0, 1, 0, 1, 0, 1, 0], 0, [], MessageType("empty"))
+            selected_message = Message([1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0], 0, [], MessageType("empty"))
             if len(self.table_model.protocol.messages) > 0:
-                selected_message.bit_len = self.table_model.protocol.messages[0].bit_len
+                selected_message.samples_per_symbol = self.table_model.protocol.messages[0].samples_per_symbol
 
         for m in self.modulators:
             m.default_sample_rate = self.project_manager.device_conf["sample_rate"]
@@ -273,7 +274,7 @@ class GeneratorTabController(QWidget):
         return modulator_dialog, selected_message
 
     def set_modulation_profile_status(self):
-        visible = constants.SETTINGS.value("multiple_modulations", False, bool)
+        visible = settings.read("multiple_modulations", False, bool)
         self.ui.cBoxModulations.setVisible(visible)
 
     def init_rfcat_plugin(self):
@@ -295,7 +296,7 @@ class GeneratorTabController(QWidget):
         modulator_dialog, message = self.prepare_modulation_dialog()
         modulator_dialog.showMaximized()
 
-        modulator_dialog.initialize(message.encoded_bits_str[0:10])
+        modulator_dialog.initialize(message.encoded_bits_str[0:16])
         self.project_manager.modulation_was_edited = True
 
     @pyqtSlot()
@@ -391,26 +392,29 @@ class GeneratorTabController(QWidget):
             except Exception as e:
                 logger.exception(e)
                 sample_rate = 1e6
-            FileOperator.save_data_dialog("generated.complex", modulated_samples, sample_rate=sample_rate, parent=self)
+            FileOperator.ask_signal_file_name_and_save("generated", modulated_samples, sample_rate=sample_rate, parent=self)
         except Exception as e:
-            Errors.generic_error(self.tr("Failed to generate data"), str(e), traceback.format_exc())
+            Errors.exception(e)
             self.unsetCursor()
 
-    def prepare_modulation_buffer(self, total_samples: int, show_error=True) -> np.ndarray:
-        memory_size_for_buffer = total_samples * 8
+    def prepare_modulation_buffer(self, total_samples: int, show_error=True) -> IQArray:
+        dtype = Modulator.get_dtype()
+        n = 2 if dtype == np.int8 else 4 if dtype == np.int16 else 8
+
+        memory_size_for_buffer = total_samples * n
         logger.debug("Allocating {0:.2f}MB for modulated samples".format(memory_size_for_buffer / (1024 ** 2)))
         try:
             # allocate it three times as we need the same amount for the sending process
-            np.zeros(3*total_samples, dtype=np.complex64)
+            IQArray(None, dtype=dtype, n=3*total_samples)
         except MemoryError:
             # will go into continuous mode in this case
             if show_error:
                 Errors.not_enough_ram_for_sending_precache(3*memory_size_for_buffer)
             return None
 
-        return np.zeros(total_samples, dtype=np.complex64)
+        return IQArray(None, dtype=dtype, n=total_samples)
 
-    def modulate_data(self, buffer: np.ndarray) -> np.ndarray:
+    def modulate_data(self, buffer: IQArray) -> IQArray:
         """
         
         :param buffer: Buffer in which the modulated data shall be written, initialized with zeros
@@ -445,9 +449,14 @@ class GeneratorTabController(QWidget):
             fdc = FuzzingDialog(protocol=self.table_model.protocol, label_index=label_index,
                                 msg_index=msg_index, proto_view=view, parent=self)
             fdc.show()
-            fdc.finished.connect(self.refresh_label_list)
-            fdc.finished.connect(self.refresh_table)
-            fdc.finished.connect(self.set_fuzzing_ui_status)
+            fdc.finished.connect(self.on_fuzzing_dialog_finished)
+
+    @pyqtSlot()
+    def on_fuzzing_dialog_finished(self):
+        self.refresh_label_list()
+        self.refresh_table()
+        self.set_fuzzing_ui_status()
+        self.ui.tabWidget.setCurrentIndex(2)
 
     @pyqtSlot()
     def handle_plabel_fuzzing_state_changed(self):
@@ -524,10 +533,10 @@ class GeneratorTabController(QWidget):
             return
 
         avg_msg_len = numpy.mean([len(msg.encoded_bits) for msg in c.messages])
-        avg_bit_len = numpy.mean([m.samples_per_bit for m in self.modulators])
+        avg_samples_per_symbol = numpy.mean([m.samples_per_symbol for m in self.modulators])
         avg_sample_rate = numpy.mean([m.sample_rate for m in self.modulators])
         pause_samples = sum(c.pauses)
-        nsamples = c.num_messages * avg_msg_len * avg_bit_len + pause_samples
+        nsamples = c.num_messages * avg_msg_len * avg_samples_per_symbol + pause_samples
 
         self.ui.lEstimatedTime.setText(
             locale.format_string("Estimated Time: %.04f seconds", nsamples / avg_sample_rate))
@@ -602,12 +611,12 @@ class GeneratorTabController(QWidget):
             dialog.show()
             dialog.graphics_view.show_full_scene(reinitialize=True)
         except Exception as e:
-            Errors.generic_error(self.tr("Failed to generate data"), str(e), traceback.format_exc())
+            Errors.exception(e)
             self.unsetCursor()
 
     @pyqtSlot()
     def on_btn_save_clicked(self):
-        filename = FileOperator.get_save_file_name("profile.fuzz.xml", caption="Save fuzz profile")
+        filename = FileOperator.ask_save_file_name("profile.fuzz.xml", caption="Save fuzzing profile")
         if filename:
             self.table_model.protocol.to_xml_file(filename,
                                                   decoders=self.project_manager.decodings,
@@ -646,6 +655,24 @@ class GeneratorTabController(QWidget):
         is_plugin_enabled = PluginManager().is_plugin_enabled("RfCat")
         self.ui.btnRfCatSend.setVisible(is_plugin_enabled)
 
+    def set_FZSave_button_visibility(self):
+        is_plugin_enabled = PluginManager().is_plugin_enabled("FlipperZeroSub")
+        self.ui.btnFZSave.setVisible(is_plugin_enabled)
+
+    @pyqtSlot()
+    def on_btn_FZSave_clicked(self):
+        filename, _ = QFileDialog.getSaveFileName(self, self.tr("Choose file"), directory=FileOperator.RECENT_PATH,
+                                                  filter="SUB files (*.sub);;All files (*.*)")
+        if len(filename) > 0:
+            if not filename.endswith(".sub"):
+                filename = filename+".sub"
+
+            messages = self.table_model.protocol.messages
+            sample_rates = [self.__get_modulator_of_message(msg).sample_rate for msg in messages]
+            self.flipperzerosub_plugin.write_sub_file(filename, messages, sample_rates, self.modulators, self.project_manager)
+        else:
+            logger.debug("Filename was empty!")
+
     @pyqtSlot()
     def on_btn_network_sdr_clicked(self):
         if not self.network_sdr_plugin.is_sending:
