@@ -96,7 +96,7 @@ class IQArray(object):
         return self.__data.tostring()
 
     def subarray(self, start=None, stop=None, step=None):
-        return IQArray(self[start:stop:step])
+        return IQArray(np.ascontiguousarray(self[start:stop:step]))
 
     def insert_subarray(self, pos, subarray: np.ndarray):
         if subarray.ndim == 1:
@@ -241,3 +241,45 @@ class IQArray(object):
         f.setframerate(sample_rate)
         f.writeframes(self.convert_to(np.int16))
         f.close()
+
+    def export_to_sub(self, filename, frequency=433920000, preset="FuriHalSubGhzPresetOok650Async"):
+        arr = []
+        counter = 0
+
+        for value in self.convert_to(np.uint8):
+            # if origin was a sub-file value is uint8, else value is [uint8, uint8]
+            if len(value) > 1:  # if value has more than one component, use the first component
+                value = value[0]
+
+            # set lastvalue to value for first run
+            try:
+                lastvalue
+            except:
+                lastvalue = value
+
+            # increase counter while values do not change
+            if value == lastvalue:
+                counter += 1
+            else:
+                # add number of same value as positive int when value > 127 else as negative int
+                if counter > 1:
+                    arr.append(counter if lastvalue > 127 else -counter)
+                    counter = 1
+                    lastvalue = value
+        # save last value
+        arr.append(counter if lastvalue > 127 else -counter)
+
+        with open(filename, 'w') as subfile:
+            subfile.write("Filetype: Flipper SubGhz RAW File\n")
+            subfile.write("Version: 1\n")
+            subfile.write("Frequency: {}\n".format(frequency))
+            subfile.write("Preset: {}\n".format(preset))
+            subfile.write("Protocol: RAW") # Skip last \n
+            # Write data
+            for idx in range(len(arr)):
+                if idx % 512 == 0:
+                    subfile.write("\n")
+                    subfile.write("RAW_Data: {}".format(arr[idx]))
+                else:
+                    subfile.write(" {}".format(arr[idx]))
+            subfile.write("\n")
\ No newline at end of file
