@@ -1,23 +1,22 @@
 import array
 import copy
-import sys
 import xml.etree.ElementTree as ET
 from xml.dom import minidom
 
 import numpy as np
 from PyQt5.QtCore import QObject, pyqtSignal, Qt
 
-from urh import constants
-from urh.awre.FormatFinder import FormatFinder
-from urh.cythonext import signal_functions, util
+from urh import settings
+from urh.cythonext import signal_functions
+import urh.dev.PCAPNG as PCAPNG
 from urh.signalprocessing.Encoding import Encoding
-from urh.signalprocessing.FieldType import FieldType
 from urh.signalprocessing.Message import Message
 from urh.signalprocessing.MessageType import MessageType
 from urh.signalprocessing.Modulator import Modulator
 from urh.signalprocessing.Participant import Participant
+from urh.signalprocessing.ProtocoLabel import ProtocolLabel
 from urh.signalprocessing.Signal import Signal
-from urh.util import util as urh_util
+from urh.util import util as urh_util, util
 from urh.util.Logger import logger
 
 
@@ -120,6 +119,10 @@ class ProtocolAnalyzer(object):
     def plain_ascii_str(self):
         return [msg.plain_ascii_str for msg in self.messages]
 
+    @property
+    def decoded_bits(self):
+        return [msg.decoded_bits for msg in self.messages]
+
     @property
     def decoded_proto_bits_str(self):
         """
@@ -164,7 +167,7 @@ class ProtocolAnalyzer(object):
 
         :param view: 0 - Bits ## 1 - Hex ## 2 - ASCII
         """
-        time = constants.SETTINGS.value('show_pause_as_time', type=bool)
+        time = settings.read('show_pause_as_time', type=bool)
         if show_pauses and time and self.signal:
             srate = self.signal.sample_rate
         else:
@@ -175,7 +178,7 @@ class ProtocolAnalyzer(object):
                                             ) for msg in self.messages)
 
     def plain_to_html(self, view, show_pauses=True) -> str:
-        time = constants.SETTINGS.value('show_pause_as_time', type=bool)
+        time = settings.read('show_pause_as_time', type=bool)
         if show_pauses and time and self.signal:
             srate = self.signal.sample_rate
         else:
@@ -185,7 +188,7 @@ class ProtocolAnalyzer(object):
         for message in self.messages:
             cur_str = ""
             if message.participant:
-                color = constants.PARTICIPANT_COLORS[message.participant.color_index]
+                color = settings.PARTICIPANT_COLORS[message.participant.color_index]
                 red, green, blue = color.red(), color.green(), color.blue()
                 fgcolor = "#000000" if (red * 0.299 + green * 0.587 + blue * 0.114) > 186 else "#ffffff"
                 cur_str += '<span style="background-color: rgb({0},{1},{2}); color: {3}">'.format(red, green, blue,
@@ -220,63 +223,59 @@ class ProtocolAnalyzer(object):
         else:
             self.messages = []
 
-        bit_len = signal.bit_len
+        samples_per_symbol = signal.samples_per_symbol
 
-        try:
-            ppseq = signal_functions.grab_pulse_lens(signal.qad, signal.qad_center, signal.tolerance,
-                                                    signal.modulation_type, signal.bit_len)
-        except TypeError:
-            # Remove this check in version 1.7
-            print("Extension method has changed! To fix this, first move to URHs base directory "
-                  "then recompile the extensions using the following command:")
-            print("python3 src/urh/cythonext/build.py")
-            print("and finally restart the application")
-            sys.exit(1)
+        ppseq = signal_functions.grab_pulse_lens(signal.qad, signal.center, signal.tolerance,
+                                                 signal.modulation_type, signal.samples_per_symbol,
+                                                 signal.bits_per_symbol, signal.center_spacing)
 
-        bit_data, pauses, bit_sample_pos = self._ppseq_to_bits(ppseq, bit_len, pause_threshold=signal.pause_threshold)
-        if signal.message_length_divisor > 1 and signal.modulation_type_str == "ASK":
-            self.__ensure_message_length_multiple(bit_data, signal.bit_len, pauses, bit_sample_pos,
+        bit_data, pauses, bit_sample_pos = self._ppseq_to_bits(ppseq, samples_per_symbol, self.signal.bits_per_symbol,
+                                                               pause_threshold=signal.pause_threshold)
+        if signal.message_length_divisor > 1 and signal.modulation_type == "ASK":
+            self.__ensure_message_length_multiple(bit_data, signal.samples_per_symbol, pauses, bit_sample_pos,
                                                   signal.message_length_divisor)
 
         i = 0
         for bits, pause in zip(bit_data, pauses):
             middle_bit_pos = bit_sample_pos[i][int(len(bits) / 2)]
-            start, end = middle_bit_pos, middle_bit_pos + bit_len
-            rssi = np.mean(np.abs(signal.data[start:end]))
+            start, end = middle_bit_pos, middle_bit_pos + samples_per_symbol
+            rssi = np.mean(signal.iq_array.subarray(start, end).magnitudes_normalized)
             message = Message(bits, pause, message_type=self.default_message_type,
-                              bit_len=bit_len, rssi=rssi, decoder=self.decoder, bit_sample_pos=bit_sample_pos[i])
+                              samples_per_symbol=samples_per_symbol, rssi=rssi, decoder=self.decoder,
+                              bit_sample_pos=bit_sample_pos[i], bits_per_symbol=signal.bits_per_symbol)
             self.messages.append(message)
             i += 1
 
         self.qt_signals.protocol_updated.emit()
 
     @staticmethod
-    def __ensure_message_length_multiple(bit_data, bit_len: int, pauses, bit_sample_pos, divisor: int):
+    def __ensure_message_length_multiple(bit_data, samples_per_symbol: int, pauses, bit_sample_pos, divisor: int):
         """
         In case of ASK modulation, this method tries to use pauses after messages as zero bits so that
         the bit lengths of messages are divisible by divisor
+
         :param bit_data: List of bit arrays
-        :param bit_len: Bit length that was used for demodulation
+        :param samples_per_symbol: Symbol length that was used for demodulation
         :param pauses: List of pauses
         :param bit_sample_pos: List of Array of bit sample positions
         :param divisor: Divisor the messages should be divisible by
         """
         for i in range(len(bit_data)):
             missing_bits = (divisor - (len(bit_data[i]) % divisor)) % divisor
-            if missing_bits > 0 and pauses[i] >= bit_len * missing_bits:
+            if missing_bits > 0 and pauses[i] >= samples_per_symbol * missing_bits:
                 bit_data[i].extend([0] * missing_bits)
-                pauses[i] = pauses[i] - missing_bits * bit_len
+                pauses[i] = pauses[i] - missing_bits * samples_per_symbol
 
                 try:
-                    bit_sample_pos[i][-1] = bit_sample_pos[i][-2] + bit_len
+                    bit_sample_pos[i][-1] = bit_sample_pos[i][-2] + samples_per_symbol
                 except IndexError as e:
                     logger.warning("Error padding message " + str(e))
                     continue
 
-                bit_sample_pos[i].extend([bit_sample_pos[i][-1] + (k + 1) * bit_len for k in range(missing_bits - 1)])
+                bit_sample_pos[i].extend([bit_sample_pos[i][-1] + (k + 1) * samples_per_symbol for k in range(missing_bits - 1)])
                 bit_sample_pos[i].append(bit_sample_pos[i][-1] + pauses[i])
 
-    def _ppseq_to_bits(self, ppseq, bit_len: int, write_bit_sample_pos=True, pause_threshold=8):
+    def _ppseq_to_bits(self, ppseq, samples_per_symbol: int, bits_per_symbol: int, write_bit_sample_pos=True, pause_threshold=8):
         bit_sampl_pos = array.array("L", [])
         bit_sample_positions = []
 
@@ -286,12 +285,12 @@ class ProtocolAnalyzer(object):
         start = 0
         total_samples = 0
 
-        pause_type = 42
-        zero_pulse_type = 0
-        one_pulse_type = 1
+        pause_type = -1
 
         there_was_data = False
 
+        samples_per_bit = int(samples_per_symbol/bits_per_symbol)
+
         if len(ppseq) > 0 and ppseq[0, 0] == pause_type:
             start = 1  # Starts with Pause
             total_samples = ppseq[0, 1]
@@ -299,19 +298,20 @@ class ProtocolAnalyzer(object):
         for i in range(start, len(ppseq)):
             cur_pulse_type = ppseq[i, 0]
             num_samples = ppseq[i, 1]
-            num_bits_floated = num_samples / bit_len
-            num_bits = int(num_bits_floated)
-            decimal_place = num_bits_floated - num_bits
+            num_symbols_float = num_samples / samples_per_symbol
+            num_symbols = int(num_symbols_float)
+            decimal_place = num_symbols_float - num_symbols
 
             if decimal_place > 0.5:
-                num_bits += 1
+                num_symbols += 1
 
             if cur_pulse_type == pause_type:
                 # OOK
-                if num_bits <= pause_threshold or pause_threshold == 0:
-                    data_bits.extend([False] * num_bits)
+                if num_symbols <= pause_threshold or pause_threshold == 0:
+                    data_bits.extend([0] * (num_symbols * bits_per_symbol))
                     if write_bit_sample_pos:
-                        bit_sampl_pos.extend([total_samples + k * bit_len for k in range(num_bits)])
+                        bit_sampl_pos.extend([total_samples + k * samples_per_bit
+                                              for k in range(num_symbols*bits_per_symbol)])
 
                 elif not there_was_data:
                     # Ignore this pause, if there were no information
@@ -330,18 +330,12 @@ class ProtocolAnalyzer(object):
                     data_bits[:] = array.array("B", [])
                     pauses.append(num_samples)
                     there_was_data = False
-
-            elif cur_pulse_type == zero_pulse_type:
-                data_bits.extend([False] * num_bits)
+            else:
+                data_bits.extend(util.number_to_bits(cur_pulse_type, bits_per_symbol) * num_symbols)
+                if not there_was_data and num_symbols > 0:
+                    there_was_data = True
                 if write_bit_sample_pos:
-                    bit_sampl_pos.extend([total_samples + k * bit_len for k in range(num_bits)])
-
-            elif cur_pulse_type == one_pulse_type:
-                if not there_was_data:
-                    there_was_data = num_bits > 0
-                data_bits.extend([True] * num_bits)
-                if write_bit_sample_pos:
-                    bit_sampl_pos.extend([total_samples + k * bit_len for k in range(num_bits)])
+                    bit_sampl_pos.extend([total_samples + k * samples_per_bit for k in range(num_symbols*bits_per_symbol)])
 
             total_samples += num_samples
 
@@ -652,6 +646,13 @@ class ProtocolAnalyzer(object):
         root = tree.getroot()
         self.from_xml_tag(root, read_bits=read_bits)
 
+    def to_pcapng(self, filename : str, hardware_desc_name: str = "", link_type: int = 147):
+        PCAPNG.create_pcapng_file(filename=filename, shb_userappl="Universal Radio Hacker", shb_hardware=hardware_desc_name, link_type=link_type)
+        PCAPNG.append_packets_to_pcapng(
+            filename=filename,
+            packets=(msg.decoded_ascii_buffer for msg in self.messages),
+            timestamps=(msg.timestamp for msg in self.messages))
+
     def eliminate(self):
         self.message_types = None
         self.messages = None
@@ -666,89 +667,15 @@ class ProtocolAnalyzer(object):
                     message.message_type = message_type
                     break
 
-    def auto_assign_participants(self, participants):
-        """
-
-        :type participants: list of Participant
-        :return:
-        """
-        if len(participants) == 0:
-            return
-
-        if len(participants) == 1:
-            for message in self.messages:
-                message.participant = participants[0]
-            return
-
-        # Try to assign participants based on SRC_ADDRESS label and participant address
-        for msg in filter(lambda m: m.participant is None, self.messages):
-            src_address = msg.get_src_address_from_data()
-            if src_address:
-                try:
-                    msg.participant = next(p for p in participants if p.address_hex == src_address)
-                except StopIteration:
-                    pass
-
-        # Assign remaining participants based on RSSI of messages
-        rssis = np.array([msg.rssi for msg in self.messages], dtype=np.float32)
-        min_rssi, max_rssi = util.minmax(rssis)
-        center_spacing = (max_rssi - min_rssi) / (len(participants) - 1)
-        centers = [min_rssi + i * center_spacing for i in range(0, len(participants))]
-        rssi_assigned_centers = []
-
-        for rssi in rssis:
-            center_index = np.argmin(np.abs(rssi - centers))
-            rssi_assigned_centers.append(int(center_index))
-
-        participants.sort(key=lambda participant: participant.relative_rssi)
-        for message, center_index in zip(self.messages, rssi_assigned_centers):
-            if message.participant is None:
-                message.participant = participants[center_index]
-
-    def auto_assign_participant_addresses(self, participants):
-        """
-
-        :type participants: list of Participant
-        :return:
-        """
-        participants_without_address = [p for p in participants if not p.address_hex]
-
-        if len(participants_without_address) == 0:
-            return
-
-        for msg in self.messages:
-            if msg.participant in participants_without_address:
-                src_address = msg.get_src_address_from_data()
-                if src_address:
-                    participants_without_address.remove(msg.participant)
-                    msg.participant.address_hex = src_address
-
-    def auto_assign_decodings(self, decodings):
-        """
-        :type decodings: list of Encoding
-        """
-        nrz_decodings = [decoding for decoding in decodings if decoding.is_nrz or decoding.is_nrzi]
-        fallback = nrz_decodings[0] if nrz_decodings else None
-        candidate_decodings = [decoding for decoding in decodings
-                               if decoding not in nrz_decodings and not decoding.contains_cut]
-
-        for message in self.messages:
-            decoder_found = False
-
-            for decoder in candidate_decodings:
-                if decoder.applies_for_message(message.plain_bits):
-                    message.decoder = decoder
-                    decoder_found = True
-                    break
-
-            if not decoder_found and fallback:
-                message.decoder = fallback
-
     def auto_assign_labels(self):
-        format_finder = FormatFinder(self)
+        from urh.awre.FormatFinder import FormatFinder
+        format_finder = FormatFinder(self.messages)
+        format_finder.run(max_iterations=10)
 
-        # OPEN: Perform multiple iterations with varying priorities later
-        format_finder.perform_iteration()
+        self.message_types[:] = format_finder.message_types
+        for msg_type, indices in format_finder.existing_message_types.items():
+            for i in indices:
+                self.messages[i].message_type = msg_type
 
     @staticmethod
     def get_protocol_from_string(message_strings: list, is_hex=None, default_pause=0, sample_rate=1e6):
@@ -767,7 +694,7 @@ class ProtocolAnalyzer(object):
 
             # support pauses given like 100101/10s
             try:
-                data, pause = line.split(constants.PAUSE_SEP)
+                data, pause = line.split(settings.PAUSE_SEP)
             except ValueError:
                 data, pause = line, str(default_pause)
             if pause.endswith("ms"):
