@@ -1,7 +1,4 @@
-from PyQt5.QtCore import QRectF
-from PyQt5.QtGui import QImage, QPixmap
-
-from urh import constants
+from urh import settings
 from urh.ui.painting.VerticalSelection import VerticalSelection
 from urh.ui.painting.ZoomableScene import ZoomableScene
 
@@ -11,7 +8,7 @@ class SpectrogramScene(ZoomableScene):
         super().__init__(parent)
         self.removeItem(self.selection_area)
 
-        self.selection_area = VerticalSelection(0, 0, 0, 0, fillcolor=constants.SELECTION_COLOR, opacity=0.6)
+        self.selection_area = VerticalSelection(0, 0, 0, 0, fillcolor=settings.SELECTION_COLOR, opacity=0.6)
         self.selection_area.setZValue(1)
         self.addItem(self.selection_area)
 
